// This is the main DLL file.

#include "stdafx.h"
#include "memory.h"

#include "Zlib.h"

const int BufSize = 16*1024;

namespace Zlib
{

System::String *ZipBase::ZLibVersion()
{
	return new System::String(zlibVersion());
}

Byte ZipBase::Compress(System::Byte src[],Level level) __gc[]
{
	uLongf len = src->Length+100;
	Bytef * dest = new Bytef[len];
	Byte __pin *p = &src[0];

    int ret = compress2(dest,&len,p,src->Length,level);
	p = 0;
	switch (ret) {
	case Z_MEM_ERROR:    throw new System::Exception("There is not enough memory.");
	case Z_BUF_ERROR:    throw new System::Exception("There is not enough room in the output buffer.");
	case Z_STREAM_ERROR: throw new System::Exception("The level parameter is invalid.");
	case Z_OK: ;
	}

	Byte buf __gc[] = new Byte __gc[len];

	p = &buf[0];
    memcpy(p,dest,len);
	p = 0;

	delete [] dest;

	return buf;
}

Byte ZipBase::Compress(System::Byte src[]) __gc[]
{
	return Compress(src,DefaultCompression);
}

void ZipBase::Uncompress(System::Byte dest[],System::Byte src[])
{
	Byte __pin *d = &dest[0];
	Byte __pin *s = &src[0];
	uLongf len = dest->Length;

	int ret = uncompress(d,&len,s,src->Length);
	d = s = 0;

	switch (ret) {
	case Z_MEM_ERROR:  throw new System::Exception("There is not enough memory.");
	case Z_BUF_ERROR:  throw new System::Exception("There is not enough room in the output buffer.");
	case Z_DATA_ERROR: throw new System::Exception("The input data was corrupted");
	case Z_OK: ;
	}
	if (dest->Length != len)
		throw new System::Exception("Output buffer lenght is incorrect");
}


ZipFile::ZipFile(System::String *fname)
{
	m_file = 0;
	Open(fname,false);
}

ZipFile::ZipFile(System::String *fname,System::Boolean append)
{
	m_file = 0;
	Open(fname,append);
}

ZipFile::~ZipFile()
{
	Close();
}

void ZipFile::Open(System::String *fname,System::Boolean append)
{
	Close();
    char bytes __gc[] = System::Text::Encoding::ASCII->GetBytes(fname);
	char __pin *p = &bytes[0];

	m_file = zipOpen(p,append!=false?1:0);
	p = 0;
	if (m_file == 0)
		throw new System::Exception("File could not be created.");
}

void ZipFile::AddFile(System::String *fname)
{
	AddFile(fname,DefaultCompression);
}

void ZipFile::AddFile(System::String *fname,Level level)
{
	int pos = fname->LastIndexOf(L'\\');
	if (pos == -1)
		pos = fname->LastIndexOf(L'/');

	AddFile(fname,pos==-1?fname:fname->Substring(pos+1),DefaultCompression);
}

void ZipFile::AddFile(System::String *fname,System::String *zipname)
{
	AddFile(fname,zipname,DefaultCompression);
}

void ZipFile::AddFile(System::String *fname,System::String *zipname,Level level)
{
	if (m_file == 0)
		throw new System::Exception("Zip file is not opened.");

	System::DateTime fdt = System::IO::File::GetCreationTime(fname);

	System::IO::FileStream *fl = 
		new System::IO::FileStream(
			fname,
			System::IO::FileMode::Open,
			System::IO::FileAccess::Read,
			System::IO::FileShare::Read);

    char bytes __gc[] = System::Text::Encoding::ASCII->GetBytes(zipname);
	char __pin *p = &bytes[0];

	zip_fileinfo zipfi;
	memset(&zipfi,0,sizeof zipfi);
	zipfi.tmz_date.tm_mon  = fdt.Month - 1;
	zipfi.tmz_date.tm_mday = fdt.Day;
	zipfi.tmz_date.tm_year = fdt.Year;
	zipfi.tmz_date.tm_hour = fdt.Hour;
	zipfi.tmz_date.tm_min  = fdt.Minute;
	zipfi.tmz_date.tm_sec  = fdt.Second;

	int ret = zipOpenNewFileInZip(
		m_file,
		p,
		&zipfi,
		0,0,
		0,0,
		0,
		Z_DEFLATED,
		level);

	char buf __gc[] = new Byte __gc[BufSize];
	p = &buf[0];
	for (__int64 n=0; ret==Z_OK && n<fl->Length; n+=BufSize) {
		OnCompress(zipname,n,fl->Length);
		__int64 sz = fl->Length - n;
		if (sz > BufSize)
			sz = BufSize;

		fl->Read(buf,0,int(sz));

        ret = zipWriteInFileInZip(m_file,p,unsigned(sz));
	}
	OnCompress(zipname,fl->Length,fl->Length);
	p = 0;
	fl->Close();

	if (ret == Z_OK)
		ret = zipCloseFileInZip(m_file);
	if (ret != Z_OK)
		throw new System::Exception("File could not be added.");
}

void ZipFile::Close()
{
	if (m_file != 0) {
		zipClose(m_file,"");
		m_file = 0;
	}
}


UnzipFile::UnzipFile(System::String *fname)
{
	m_iscurrent = false;
    char bytes __gc[] = System::Text::Encoding::ASCII->GetBytes(fname);
	char __pin *p = &bytes[0];

	m_file = unzOpen(p);
	p = 0;
	CheckForFile();
}

UnzipFile::~UnzipFile()
{
	Close();
}

void UnzipFile::CheckForCurrent()
{
	if (m_iscurrent == false)
		throw new System::Exception("There is no current file.");
}

void UnzipFile::CheckForFile()
{
	if (m_file == 0)
		throw new System::Exception("Zip file is not opened.");
}

void UnzipFile::Close()
{
	if (m_file) {
        unzClose(m_file);
		m_file = 0;
		m_iscurrent = false;
	}
}

bool UnzipFile::First()
{
	CheckForFile();
	return m_iscurrent = UNZ_OK == unzGoToFirstFile(m_file);
}

bool UnzipFile::Next()
{
	CheckForFile();
	return m_iscurrent = UNZ_OK == unzGoToNextFile(m_file);
}

bool UnzipFile::Locate(System::String *fname)
{
	return Locate(fname,false);
}

bool UnzipFile::Locate(System::String *fname,bool CaseSensitivity)
{
	CheckForFile();

	char bytes __gc[] = System::Text::Encoding::ASCII->GetBytes(fname);
	char __pin *p = &bytes[0];

	m_iscurrent = UNZ_OK == unzLocateFile(m_file,p,CaseSensitivity);
	p = 0;

	return m_iscurrent;
}

ZipInfo __gc *UnzipFile::GetInfo()
{
	CheckForCurrent();

	ZipInfo __gc *fi = new ZipInfo;

	unz_file_info_s zfi;

	char fname[300];
	int ret = unzGetCurrentFileInfo(
		m_file,
		&zfi,
		fname,
		sizeof(fname)-1,
		0,0,
		0,0);

	if (ret != UNZ_OK)
		throw new System::Exception("GetInfo error.");

	fi->Version           = zfi.version;
	fi->VersionNeeded     = zfi.version_needed;
	fi->Flag              = zfi.flag;
	fi->CompressionMethod = zfi.compression_method;
	fi->CRC               = zfi.crc;
	fi->CompressedSize    = zfi.compressed_size;
	fi->UncompressedSize  = zfi.uncompressed_size;

	fi->FileName = fname;
	fi->Date     = System::DateTime(
					zfi.tmu_date.tm_year,
					zfi.tmu_date.tm_mon,
					zfi.tmu_date.tm_mday,
					zfi.tmu_date.tm_hour,
					zfi.tmu_date.tm_min,
					zfi.tmu_date.tm_sec);

	return fi;
}

void UnzipFile::SaveFile(System::String *fname)
{
	ZipInfo *zi = GetInfo();

	System::String *fn = fname;

	if (fname->Length == 0 || 
		fname->Chars[fname->Length-1] == L'\\' ||
		fname->Chars[fname->Length-1] == L'/')
	{
		fn = fn->Concat(fn,zi->FileName);
	}

	System::IO::FileStream *file = 
		new System::IO::FileStream(
			fn,
			System::IO::FileMode::Create,
			System::IO::FileAccess::Write);

	int ret = unzOpenCurrentFile(m_file);

	if (ret == UNZ_OK) {
		Byte buf __gc[] = new Byte __gc[BufSize];
		Byte __pin *p = &buf[0];

		for (__int64 n=0;;n+=ret) {
			OnUncompress(zi->FileName,n,zi->UncompressedSize);
			ret = unzReadCurrentFile(m_file,p,BufSize);
			if (ret <= 0)
				break;
			file->Write(buf,0,ret);
		}
        unzCloseCurrentFile(m_file);
		OnUncompress(zi->FileName,zi->UncompressedSize,zi->UncompressedSize);
	}

	file->Close();
	System::IO::File::SetCreationTime(fn,zi->Date);
	System::IO::File::SetLastWriteTime(fn,zi->Date);

	if (ret != UNZ_OK)
		throw new System::Exception("Could not save file.");
}

}

#include <vcclr.h>
