// Zlib.h

#pragma once

//using namespace System;
#include "Zlib\Zlib.h"
#include "Zlib\Contrib\Minizip\Zip.h"
#include "Zlib\Contrib\Minizip\Unzip.h"

namespace Zlib
{

public __delegate void CompressHandler  (System::String *name,System::UInt64 sz,System::UInt64 cur);
public __delegate void UncompressHandler(System::String *name,System::UInt64 sz,System::UInt64 cur);

public __gc class ZipBase {
public:
	__value enum Level : int {
		NoCompression = Z_NO_COMPRESSION,
		BestSpeed = Z_BEST_SPEED,
		BestCompression = Z_BEST_COMPRESSION,
		DefaultCompression = Z_DEFAULT_COMPRESSION
	};

	static System::String *ZLibVersion();
	static Byte Compress(System::Byte src[]) __gc[];
	static Byte Compress(System::Byte src[],Level level) __gc[];
	static void Uncompress(System::Byte dest[],System::Byte src[]);
};

public __gc class ZipFile : public ZipBase {
public:
	ZipFile(System::String*);
	ZipFile(System::String*,System::Boolean append);
	~ZipFile();

	__event CompressHandler *OnCompress;

	void AddFile(System::String *fname);
	void AddFile(System::String *fname,Level);
	void AddFile(System::String *fname,System::String *zipname);
	void AddFile(System::String *fname,System::String *zipname,Level);

	void Close();
private:
	zipFile m_file;
	void Open(System::String *fname,System::Boolean append);
};

public __gc struct ZipInfo {
	unsigned Version;	        // version made by
	unsigned VersionNeeded;     // version needed to extract
	unsigned Flag;              // general purpose bit flag
	unsigned CompressionMethod;	// compression method
	unsigned CRC;               // crc-32
	unsigned CompressedSize;    // compressed size
	unsigned UncompressedSize;  // uncompressed size

	System::DateTime Date;
	System::String  *FileName;
};

public __gc class UnzipFile : public ZipBase {
public:
	UnzipFile(System::String *fname);
	~UnzipFile();

	__event UncompressHandler *OnUncompress;

	void Close();
	
	bool First();
	bool Next();
	bool Locate(System::String *fname);
	bool Locate(System::String *fname,bool CaseSensitivity);

	ZipInfo __gc *GetInfo();

	void SaveFile(System::String *fname);
private:
	unzFile m_file;
	bool m_iscurrent;
	void CheckForCurrent();
	void CheckForFile();
};

}
